/*
 *   Copyright (C) 2009-2011 Yury P. Fedorchenko
 *   yuryfdr@users.sf.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */
#include <FL/Fl.H>
#include <FL/Fl_Double_Window.H>

#include "flothello.h"
#include <FL/Fl_Menu_Bar.H>
#include <FL/Fl_Choice.H>
#include <FL/Fl_Output.H>
#include <FL/Fl_Check_Button.H>
#include <FL/Fl_Radio_Round_Button.H>
#include <FL/Fl_Return_Button.H>
#include <FL/fl_draw.H>
#include <FL/Fl_Pixmap.H>
#include <FL/fl_ask.H>
#include <FL/Fl_Box.H>

#include <iostream>
using std::endl;
using std::cerr;

void Fl_RevBoard::draw(){
    draw_box();
    int item=0;
    for(int j=0;j<8;++j){
      for(int i=0;i<8;++i){
        fl_color(FL_WHITE);
        int X=x()+i*csize
           ,Y=y()+j*csize;
        fl_rect(X,Y,csize,csize);
        if(drawboard[i][j])pixmaps->draw(X,Y,csize,csize,csize*drawboard[i][j],0);
      }
    }
}

Fl_RevBoard* Fl_RevBoard::brd;
void Fl_RevBoard::flip_animate(){
  if(brd->board.game_in_progress==0){
    brd->board.animate_stagger=29;
    Fl::add_timeout(0.01,(Fl_Timeout_Handler)animate,brd);
  }else if(brd->board.whose_turn == rev_board::WHITE_TURN){
    brd->board.animate_stagger=29;
    Fl::add_timeout(0.01,(Fl_Timeout_Handler)animate,brd);
  }else{
    brd->board.animate_stagger=-30;
    Fl::add_timeout(0.01,(Fl_Timeout_Handler)animate,brd);
  }
}

void Fl_RevBoard::animate(Fl_RevBoard* b){
  b->copyboard(true);  
  b->redraw();
  Fl::check();
  if(b->board.animate_stagger!=0){
    Fl::repeat_timeout(0.01,(Fl_Timeout_Handler)animate,b);
  }else{
    b->copyboard(false);
    b->redraw();
    Fl::check();
    Fl::remove_timeout((Fl_Timeout_Handler)animate);
    brd->board.check_valid_moves();
    brd->board.check_computer_players();
  }
}

int Fl_RevBoard::handle(int e){
  if(FL_PUSH == e && !board.animate_stagger){
    int X=(Fl::event_x()-x())/csize;
    int Y=(Fl::event_y()-y())/csize;
    if (board.is_valid_move (X, Y, board.whose_turn) ){
      board.move(X, Y, board.whose_turn);
    }
  }
}

Fl_Output* out;
Fl_Box* status;

int Fl_RevBoard::gui_message(const char* str){
  out->value(str);
}
void Fl_RevBoard::gui_status(){
  static char str[128];
  sprintf(str,_("black:%.2d white:%.2d"), brd->board.bcount, brd->board.wcount); 
  if (brd->board.whose_turn == rev_board::BLACK_TURN) {
    gui_message (_("Dark's move"));
  } else if (brd->board.whose_turn == rev_board::WHITE_TURN) {
    gui_message (_("Light's move"));
  }
  status->label(str);
  status->redraw();
}

Fl_RevBoard* board;

void new_game(Fl_Widget*,void*){
  board->new_game();
  board->gui_status();
  board->window()->redraw();
}

void cb_exit(Fl_Widget*,void*){
  board->window()->hide();
}
void cb_light(Fl_Widget*,long p){
  board->board.white_computer_level=p;
}
void cb_dark(Fl_Widget*,long p){
  board->board.black_computer_level=p;
}

void cb_close(Fl_Widget*o,void*){
  o->window()->hide();
}

Fl_Window *opt_wnd=(Fl_Window *)0;
void cb_options(Fl_Widget*,void*){
  if(!opt_wnd){
    opt_wnd = new Fl_Window(320,240,"Options");
    Fl_Group *lgr= new Fl_Group(0,25,160,100,"Lights");
    lgr->begin();
    {Fl_Round_Button *b0 = new Fl_Radio_Round_Button(0,25,160,25,"Human");
    b0->callback(cb_light,0);
    Fl_Round_Button *b1 = new Fl_Radio_Round_Button(0,50,160,25,"Level one");
    b1->callback(cb_light,1);
    Fl_Round_Button *b2 = new Fl_Radio_Round_Button(0,75,160,25,"Level two");
    b2->callback(cb_light,2);
    Fl_Round_Button *b3 = new Fl_Radio_Round_Button(0,100,160,25,"Level three");
    b3->callback(cb_light,3);}
    lgr->end();
    ((Fl_Round_Button *)lgr->child(board->board.white_computer_level))->value(1);
    Fl_Group *rgr= new Fl_Group(160,25,160,100,"Darks");
    rgr->begin();
    {Fl_Round_Button *b0r = new Fl_Radio_Round_Button(160,25,160,25,"Human");
    b0r->callback(cb_dark,0);
    Fl_Round_Button *b1r = new Fl_Radio_Round_Button(160,50,160,25,"Level one");
    b1r->callback(cb_dark,1);
    Fl_Round_Button *b2r = new Fl_Radio_Round_Button(160,75,160,25,"Level two");
    b2r->callback(cb_dark,2);
    Fl_Round_Button *b3r = new Fl_Radio_Round_Button(160,100,160,25,"Level three");
    b3r->callback(cb_dark,3);}
    rgr->end();
    ((Fl_Round_Button *)rgr->child(board->board.black_computer_level))->value(1);
    (new Fl_Button(110,200,100,25,"Close"))->callback(cb_close);
    opt_wnd->end();
    opt_wnd->set_modal();
  }
  opt_wnd->show();
}

void cb_about(Fl_Widget*o,void*){
  fl_message("flreversi v1.0.2\nYury P. Fedorchenko 2009-2011\nwww.fedorchenko.net");
}

Fl_Menu_Item menu[]={
  {"Game",0,0,0,FL_SUBMENU},
  {"New",0,new_game},
  {"Exit",0,cb_exit},
  {0},
  {"Options",0,cb_options},
  //{0},
  {"About",0,cb_about},
  {0},
{0}
};

int main(int argc,char** argv){
  Fl_Double_Window a(404,404+25+25+25,"Flreversi");
  Fl_Menu_Bar bar(2,0,400,25);
  bar.menu(menu);
  board = new Fl_RevBoard(2,25);
  out = new Fl_Output(2,427,400,25);
  status = new Fl_Box(2,452,400,25);
  a.show(argc,argv);
  return Fl::run();
}

