/*
 *   Copyright (C) 2009-2011 Yury P. Fedorchenko
 *   yuryfdr@users.sf.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#include "pbothello.h"


ifont* boardf;
extern ibitmap classic;
extern ibitmap newp;

int pb_RevBoard::x2i(int i){
  int ret=(i-xb)/csize;
  if(ret <0 || ret>=8)return -1;
  return ret;
}
int pb_RevBoard::y2j(int i){
  int ret=(i-yb)/csize;
  if(ret <0 || ret>=8)return -1;
  return ret;
}

void pb_RevBoard::draw_item(int i,int j){
  int X=xb+i*csize;
  int Y=yb+j*csize;
  FillArea(X,Y,csize,csize,WHITE);
  DrawRect(X,Y,csize,csize,BLACK);
  DrawRect(X+1,Y+1,csize-2,csize-2,(curs_x==i&&curs_y==j)?BLACK:WHITE);
  DrawRect(X+2,Y+2,csize-4,csize-4,(curs_x==i&&curs_y==j)?BLACK:WHITE);
  if(drawboard[i][j]){
    DrawBitmapArea(X,Y,pixmaps,csize*drawboard[i][j],0,csize,csize);
  }
}

void pb_RevBoard::draw(){
    for(int j=0;j<8;++j){
      for(int i=0;i<8;++i){
        draw_item(i,j);
      }
    }
}


pb_RevBoard* pb_RevBoard::brd;
void pb_RevBoard::flip_animate(){
  if(brd->board.whose_turn == rev_board::WHITE_TURN){
    brd->board.animate_stagger=5;
    SetWeakTimer("btimer",animate,1);
  }else{
    brd->board.animate_stagger=-6;
    SetWeakTimer("btimer",animate,1);
  }
}

void pb_RevBoard::animate(){
  printf("AS:%d WT:%d\n",brd->board.animate_stagger,brd->board.whose_turn);
  brd->copyboard(true);  
  brd->draw();
  PartialUpdateBW(brd->xb,brd->yb,8*65,8*65);
  if(brd->board.animate_stagger!=0){
    SetWeakTimer("btimer",animate,10);
  }else{
    brd->copyboard(false);
    brd->draw();
    SoftUpdate();
    ClearTimer(animate);
    brd->board.check_valid_moves();
    brd->board.check_computer_players();
  }
}

void pb_RevBoard::click(int i,int j){
  curs_x=i;
  curs_y=j;
  click();
}
void pb_RevBoard::click(){
  if(!board.animate_stagger){
    int X=curs_x;
    int Y=curs_y;
    if (board.is_valid_move (X, Y, board.whose_turn) ){
      board.move(X, Y, board.whose_turn);
    }
  }
}

//Fl_Output* out;
void pb_RevBoard::gui_status(){
  SetFont(boardf,BLACK);
  FillArea(brd->xb,brd->yb+csize*9,csize*8,30,WHITE);
  static char str[128];
  sprintf(str,_("black:%.2d white:%.2d"), brd->board.bcount, brd->board.wcount); 
  DrawString(brd->xb,brd->yb+csize*9,str);
  PartialUpdateBW(brd->xb,brd->yb+csize*9,csize*8,30);
  if (brd->board.whose_turn == rev_board::BLACK_TURN) {
    gui_message (_("Dark's move"));
  } else if (brd->board.whose_turn == rev_board::WHITE_TURN) {
    gui_message (_("Light's move"));
  }
}
int pb_RevBoard::gui_message(const char* str){
  SetFont(boardf,BLACK);
  FillArea(brd->xb,brd->yb+csize*8,csize*8,csize,WHITE);
  DrawString(brd->xb,brd->yb+csize*8,str);
  PartialUpdateBW(brd->xb,brd->yb+csize*8,csize*8,csize);
  return 0;
}
void pb_RevBoard::cursor_down(){
  int co=curs_y;
  ++curs_y;
  if(curs_y==bsize)curs_y=0;
  draw_item(curs_x,co);
  draw_item(curs_x,curs_y);
  if(curs_y==0)PartialUpdateBW(xb+curs_x*csize,yb,csize,csize*bsize);
  else PartialUpdateBW(xb+curs_x*csize,yb+(curs_y-1)*csize,csize,csize*2);
}
void pb_RevBoard::cursor_up(){
  int co=curs_y;
  if(curs_y==0)curs_y=bsize;
  --curs_y;
  draw_item(curs_x,co);
  draw_item(curs_x,curs_y);
  if(curs_y==bsize-1)PartialUpdateBW(xb+curs_x*csize,yb,csize,csize*bsize);
  else PartialUpdateBW(xb+curs_x*csize,yb+curs_y*csize,csize,csize*2);
}
void pb_RevBoard::cursor_left(){
  int co=curs_x;
  if(curs_x==0)curs_x=bsize;
  --curs_x;
  draw_item(co,curs_y);
  draw_item(curs_x,curs_y);
  if(curs_x==bsize-1)PartialUpdateBW(xb,yb+curs_y*csize,csize*bsize,csize);
  else PartialUpdateBW(xb+curs_x*csize,yb+curs_y*csize,csize*2,csize);
}
void pb_RevBoard::cursor_right(){
  int co=curs_x;
  ++curs_x;
  if(curs_x==bsize)curs_x=0;
  draw_item(co,curs_y);
  draw_item(curs_x,curs_y);
  if(curs_x==0)PartialUpdateBW(xb,yb+curs_y*csize,csize*bsize,csize);
  else PartialUpdateBW(xb+(curs_x-1)*csize,yb+curs_y*csize,csize*2,csize);
}

pb_RevBoard board;

void board_repaint(const bool full=true){
  ClearScreen();
  board.draw();
  if(full)FullUpdate();
  else SoftUpdate();
}

static iconfig *revcfg = NULL;

static char *player_v[] = {"Human", "Level one", "Level two", "Level three", NULL };
static char *yesno_v[]    = {"@Yes","@No",NULL};

static iconfigedit revcfge[] = {
//int type;const ibitmap *icon;char *text;char *hint;char *name;char *deflt;char **variants;struct iconfigedit_s *submenu;
//{CFG_INFO,NULL               ,""          ,NULL      ,""        ,""         ,NULL           ,NULL},
  {CFG_INDEX,NULL             ,"Light"      ,NULL     ,"rv.light",  "1"       ,player_v       ,NULL},
  {CFG_INDEX,NULL             ,"Dark"       ,NULL     ,"rv.dark",   "0"       ,player_v       ,NULL},
  {CFG_INDEX,NULL             ,"Animate"    ,NULL     ,"rv.animate","0"       ,yesno_v        ,NULL},
  {CFG_INDEX,NULL             ,"Show Grid"  ,NULL     ,"rv.grid",   "0"       ,yesno_v        ,NULL},
  {0}
}; 


int main_handler(int, int, int);
void menu1_handler(int index);
int cindex=0;

static imenu menu1[] = {

  { ITEM_HEADER,   0, "@Menu", NULL },
  { ITEM_ACTIVE, 101, "@New Game", NULL },
  { ITEM_SEPARATOR, 0, NULL, NULL },
  { ITEM_ACTIVE, 102, "@Configuration", NULL },
  { ITEM_SEPARATOR, 0, NULL, NULL },
  { ITEM_ACTIVE, 104, "@About", NULL },
  { ITEM_SEPARATOR, 0, NULL, NULL },
  { ITEM_ACTIVE, 121, "@Exit", NULL },
  { 0, 0, NULL, NULL }

};


int set_field_handler(int type, int par1, int par2);

void apply_config(){
  board.board.white_computer_level=ReadInt(revcfg,"rv.light",1);
  board.board.black_computer_level=ReadInt(revcfg,"rv.dark",0);
  board._animate=!ReadInt(revcfg,"rv.animate",0);
  board.board.game_in_progress=0;
}

static void config_ok() {
  SaveConfig(revcfg);
  if(revcfg->changed){
    apply_config();
  }
  ClearScreen();
  SendEvent(set_field_handler,EVT_SHOW,0,0);
}


void dialog_exit_handler(int button) {
  if(1 == button)CloseApp();
}


int set_field_handler(int type, int par1, int par2)
{
  if ( EVT_SHOW == type ){
    board_repaint(false);
  }
  if (type == EVT_KEYPRESS || type == EVT_KEYREPEAT) {
    switch (par1) {
    case KEY_MENU:
      OpenMenu(menu1, cindex, 20, 20, menu1_handler);
      return 1;
    case KEY_LEFT:
      board.cursor_left();
      break;
    case KEY_RIGHT:
      board.cursor_right();
      break;
    case KEY_UP:
      board.cursor_up();
      break;
    case KEY_DOWN:
      board.cursor_down();
      break;
    case KEY_OK:
      if(type == EVT_KEYREPEAT){
        OpenMenu(menu1, cindex, 20, 20, menu1_handler);
        return 1;
      }else{
        board.click();
        if(board.game_over())SetEventHandler(main_handler);
        return 1;
      }
      break;
    case KEY_BACK://??
      if(type == EVT_KEYREPEAT){
        OpenMenu(menu1, cindex, 20, 20, menu1_handler);
        return 1;
      }
      break;
    }
    return 1;
  }
  if(type == EVT_POINTERDOWN){
    int i = board.x2i(par1);
    if( i>=0 ){
      int j = board.y2j(par2);
      if( j>=0 ){
        board.click(i,j);
        if(board.game_over())SetEventHandler(main_handler);
        return 1;
      }else{
        OpenMenu(menu1, cindex, par1, par2, menu1_handler);
      }
    }
  }
  return 0;
}


void menu1_handler(int index) {
  cindex=index;
  switch (index) {
    case 101:
      board.new_game();
      SetEventHandler(set_field_handler);
      break;
    case 102:
      OpenConfigEditor("@Configuration", revcfg, revcfge, config_ok, NULL);
      break;
    case 104:
      Message(ICON_INFORMATION, _("About"),"PocketBook Reversi game 1.0.2\n"
        "by Yury P. Fedorchenko 2009-2011.\n"
        "some parts of code based on code fron gnome games.\n"
        "This is free sowtware and distributed under terms\n of "
        "GNU GPL License\n"
        "www.fedorchenko.net", 10000);
      break;
    case 121:
      Dialog(ICON_QUESTION, "@Exit", _("Quit game?"), "@Yes", "@No", dialog_exit_handler);
      break;
  }
  FullUpdate();
}


int main_handler(int type, int par1, int par2)
{
  if (type == EVT_INIT) {
    boardf=OpenFont("cour",24,1);
    revcfg = OpenConfig(FLASHDIR "/reversi.cfg", revcfge);
    //board.pixmaps = &classic;
    board.pixmaps = &newp;
    apply_config();
  }

  if (type == EVT_SHOW) {
    ClearScreen();
    board_repaint();
  }

  if (type == EVT_KEYPRESS || type == EVT_KEYREPEAT ) {
    switch (par1) {

      case KEY_OK:
      case KEY_MENU:
      case KEY_BACK:
        OpenMenu(menu1, cindex, 20, 20, menu1_handler);
        return 1;

    }
  }
  if(type == EVT_POINTERDOWN){
        OpenMenu(menu1, cindex, 20, 20, menu1_handler);
  }
  if (type == EVT_EXIT) {
    // occurs only in main handler when exiting or when SIGINT received.
    // save configuration here, if needed
  }

  return 0;
}

int main(int argc,char** argv){
  InkViewMain(main_handler);
  return 0;
}

