/*
 *   Copyright (C) 2009-2011 Yury P. Fedorchenko
 *   yuryfdr@users.sf.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */
#ifndef __rev_board_h__
#define __rev_board_h__

#ifdef PBBUILD
#include <inkview.h>
#define _(a) GetLangText(a)
#else
#define _(a) (a)
#endif

typedef int (msgf)(const char*);
typedef void (animatef)();

class rev_board{
protected:
  //int pixmaps[8][8];
public:
  enum game_type {OTHELLO=1,ATAXX=2} type;
  int black_computer_level;
  int white_computer_level;
  int game_in_progress;
  int board[8][8];
  int whose_turn;
protected:
  int move_count;
  static int squares[64];
public:
  int bcount;
  int wcount;
  static const int BLACK_TURN = 1;
#ifdef PBBUILD  
  static const int WHITE_TURN = 7;
#else
  static const int WHITE_TURN = 31;
#endif
  int animate_stagger;
  msgf *msg;
  animatef *flip_animate,*gui_status;
private:
  int gui_message(const char*str);
  int OTHER_PLAYER(int w){ return (((w) == WHITE_TURN) ? BLACK_TURN : WHITE_TURN);}

  void move_board0 (int xy, int dir);
  int eval_heuristic (void);
  int mobility();
  int around0 (int xy);
  int around ();
  int b_evaluation (void);
  int v_evaluation (void);
  int p_evaluation (void);
  int w_evaluation (void);
  int search (int n, int a, int b);
  void random_select();
  int computer_move(int level);
  int is_valid0 (int xy, int dir, int who);
  int is_valid (int xy, int who);
public:
  int is_valid_move (unsigned int, unsigned int, unsigned int);
  int move (unsigned int x, unsigned int y, unsigned int me);
  void move_board (int xy);
//was be bool
  void flip_final_results(void* data);
  int check_valid_moves();
  void check_computer_players();
  //int computer_move(unsigned int);
  
  void init (void);
  void board_copy (void);
  void undo (void);
  
  rev_board():black_computer_level(0),white_computer_level(1){};

  virtual ~rev_board(){};
};
class RevBoard {
public:
  bool _animate;
  int drawboard[8][8];
  rev_board board;
  bool game_over(){return (board.game_in_progress==0);}
  void new_game(){
    board.init();
    board.animate_stagger=0;
    copyboard(false);
  }
  void copyboard(bool animate){
    for(int i=0;i<8;++i){
      for(int j=0;j<8;++j){
        if(!_animate || !animate){
          drawboard[i][j]=board.board[i][j];
        }else if(board.game_in_progress==0){
          if(drawboard[i][j]!=board.board[i][j]){
            if(drawboard[i][j]==0 || board.board[i][j]==0)drawboard[i][j]=board.board[i][j];
            else if(drawboard[i][j]<board.board[i][j])
              drawboard[i][j]=board.board[i][j]-board.animate_stagger;
            else drawboard[i][j]=board.board[i][j]+board.animate_stagger;          
          }
        }else{
          if(drawboard[i][j]!=board.board[i][j]){
            if(drawboard[i][j]==0)drawboard[i][j]=board.board[i][j];
            else{
              drawboard[i][j]=board.board[i][j]+board.animate_stagger;
            }
          }
        }
      }
    }
    if(!_animate || !animate){
      board.animate_stagger=0; return;
    }
    if(board.animate_stagger>0)--board.animate_stagger;
    else if(board.animate_stagger<0)++board.animate_stagger;
  };
  virtual void draw()=0;
  virtual ~RevBoard(){};  
};
#endif //__rev_board_h__

